@@ -13,8 +13,8 @@ impl Generator for Fig {
         format!("{}.ts", name)
     }
 
-    fn generate(&self, app: &App, buf: &mut dyn Write) {
-        let command = app.get_bin_name().unwrap();
+    fn generate(&self, cmd: &Command, buf: &mut dyn Write) {
+        let command = cmd.get_bin_name().unwrap();
         let mut buffer = String::new();
 
         buffer.push_str(&format!(
@@ -24,10 +24,10 @@ impl Generator for Fig {
 
         buffer.push_str(&format!(
             "  description: \"{}\",\n",
-            app.get_about().unwrap_or_default()
+            cmd.get_about().unwrap_or_default()
         ));
 
-        gen_fig_inner(command, &[], 2, app, &mut buffer);
+        gen_fig_inner(command, &[], 2, cmd, &mut buffer);
 
         buffer.push_str("};\n\nexport default completion;\n");
 
@@ -45,13 +45,13 @@ fn gen_fig_inner(
     root_command: &str,
     parent_commands: &[&str],
     indent: usize,
-    app: &App,
+    cmd: &Command,
     buffer: &mut String,
 ) {
-    if app.has_subcommands() {
+    if cmd.has_subcommands() {
         buffer.push_str(&format!("{:indent$}subcommands: [\n", "", indent = indent));
         // generate subcommands
-        for subcommand in app.get_subcommands() {
+        for subcommand in cmd.get_subcommands() {
             buffer.push_str(&format!(
                 "{:indent$}{{\n{:indent$}  name: \"{}\",\n",
                 "",
@@ -69,6 +69,14 @@ fn gen_fig_inner(
                 ));
             }
 
+            if subcommand.is_hide_set() {
+                buffer.push_str(&format!(
+                    "{:indent$}hidden: true,\n",
+                    "",
+                    indent = indent + 4
+                ))
+            }
+
             let mut parent_commands: Vec<_> = parent_commands.into();
             parent_commands.push(subcommand.get_name());
             gen_fig_inner(
@@ -84,9 +92,17 @@ fn gen_fig_inner(
         buffer.push_str(&format!("{:indent$}],\n", "", indent = indent));
     }
 
-    buffer.push_str(&gen_options(app, indent));
+    buffer.push_str(&gen_options(cmd, indent));
 
-    let args = app.get_positionals().collect::<Vec<_>>();
+    let args = cmd.get_positionals().collect::<Vec<_>>();
+
+    if args.iter().any(|&x| x.is_require_equals_set()) {
+        buffer.push_str(&format!(
+            "{:indent$}requireEquals: true,\n",
+            "",
+            indent = indent
+        ));
+    }
 
     match args.len() {
         0 => {}
@@ -98,121 +114,192 @@ fn gen_fig_inner(
         _ => {
             buffer.push_str(&format!("{:indent$}args: [\n", "", indent = indent));
             for arg in args {
-                buffer.push_str(&gen_args(arg, indent));
+                buffer.push_str(&format!("{:indent$}", "", indent = indent + 2));
+                buffer.push_str(&gen_args(arg, indent + 2));
             }
             buffer.push_str(&format!("{:indent$}]\n", "", indent = indent));
         }
     };
 }
 
-fn gen_options(app: &App, indent: usize) -> String {
+fn gen_options(cmd: &Command, indent: usize) -> String {
     let mut buffer = String::new();
 
-    buffer.push_str(&format!("{:indent$}options: [\n", "", indent = indent));
+    let flags = generator::utils::flags(cmd);
 
-    for option in app.get_opts() {
-        buffer.push_str(&format!("{:indent$}{{\n", "", indent = indent + 2));
+    if cmd.get_opts().next().is_some() || !flags.is_empty() {
+        buffer.push_str(&format!("{:indent$}options: [\n", "", indent = indent));
 
-        let mut names = vec![];
+        for option in cmd.get_opts() {
+            buffer.push_str(&format!("{:indent$}{{\n", "", indent = indent + 2));
 
-        if let Some(shorts) = option.get_short_and_visible_aliases() {
-            names.extend(shorts.iter().map(|short| format!("-{}", short)));
+            let mut names = vec![];
+
+            if let Some(shorts) = option.get_short_and_visible_aliases() {
+                names.extend(shorts.iter().map(|short| format!("-{}", short)));
+            }
+
+            if let Some(longs) = option.get_long_and_visible_aliases() {
+                names.extend(longs.iter().map(|long| format!("--{}", long)));
+            }
+
+            if names.len() > 1 {
+                buffer.push_str(&format!("{:indent$}name: [", "", indent = indent + 4));
+
+                buffer.push_str(
+                    &names
+                        .iter()
+                        .map(|name| format!("\"{}\"", name))
+                        .collect::<Vec<_>>()
+                        .join(", "),
+                );
+
+                buffer.push_str("],\n");
+            } else {
+                buffer.push_str(&format!(
+                    "{:indent$}name: \"{}\",\n",
+                    "",
+                    names[0],
+                    indent = indent + 4
+                ));
+            }
+
+            if let Some(data) = option.get_help() {
+                buffer.push_str(&format!(
+                    "{:indent$}description: \"{}\",\n",
+                    "",
+                    escape_string(data),
+                    indent = indent + 4
+                ));
+            }
+
+            if option.is_hide_set() {
+                buffer.push_str(&format!(
+                    "{:indent$}hidden: true,\n",
+                    "",
+                    indent = indent + 4
+                ))
+            }
+
+            let conflicts = arg_conflicts(cmd, option);
+
+            if !conflicts.is_empty() {
+                buffer.push_str(&format!(
+                    "{:indent$}exclusiveOn: [\n",
+                    "",
+                    indent = indent + 4
+                ));
+
+                for conflict in conflicts {
+                    buffer.push_str(&format!(
+                        "{:indent$}\"{}\",\n",
+                        "",
+                        conflict,
+                        indent = indent + 6
+                    ));
+                }
+
+                buffer.push_str(&format!("{:indent$}],\n", "", indent = indent + 4));
+            }
+
+            if option.is_multiple_occurrences_set() {
+                buffer.push_str(&format!(
+                    "{:indent$}isRepeatable: true,\n",
+                    "",
+                    indent = indent + 4
+                ));
+            }
+
+            buffer.push_str(&format!("{:indent$}args: ", "", indent = indent + 4));
+
+            buffer.push_str(&gen_args(option, indent + 4));
+
+            buffer.push_str(&format!("{:indent$}}},\n", "", indent = indent + 2));
         }
 
-        if let Some(longs) = option.get_long_and_visible_aliases() {
-            names.extend(longs.iter().map(|long| format!("--{}", long)));
+        for flag in generator::utils::flags(cmd) {
+            buffer.push_str(&format!("{:indent$}{{\n", "", indent = indent + 2));
+
+            let mut flags = vec![];
+
+            if let Some(shorts) = flag.get_short_and_visible_aliases() {
+                flags.extend(shorts.iter().map(|s| format!("-{}", s)));
+            }
+
+            if let Some(longs) = flag.get_long_and_visible_aliases() {
+                flags.extend(longs.iter().map(|s| format!("--{}", s)));
+            }
+
+            if flags.len() > 1 {
+                buffer.push_str(&format!("{:indent$}name: [", "", indent = indent + 4));
+
+                buffer.push_str(
+                    &flags
+                        .iter()
+                        .map(|name| format!("\"{}\"", name))
+                        .collect::<Vec<_>>()
+                        .join(", "),
+                );
+
+                buffer.push_str("],\n");
+            } else {
+                buffer.push_str(&format!(
+                    "{:indent$}name: \"{}\",\n",
+                    "",
+                    flags[0],
+                    indent = indent + 4
+                ));
+            }
+
+            if let Some(data) = flag.get_help() {
+                buffer.push_str(&format!(
+                    "{:indent$}description: \"{}\",\n",
+                    "",
+                    escape_string(data).as_str(),
+                    indent = indent + 4
+                ));
+            }
+
+            let conflicts = arg_conflicts(cmd, &flag);
+
+            if !conflicts.is_empty() {
+                buffer.push_str(&format!(
+                    "{:indent$}exclusiveOn: [\n",
+                    "",
+                    indent = indent + 4
+                ));
+
+                for conflict in conflicts {
+                    buffer.push_str(&format!(
+                        "{:indent$}\"{}\",\n",
+                        "",
+                        conflict,
+                        indent = indent + 6
+                    ));
+                }
+
+                buffer.push_str(&format!("{:indent$}],\n", "", indent = indent + 4));
+            }
+
+            if flag.is_multiple_occurrences_set() {
+                buffer.push_str(&format!(
+                    "{:indent$}isRepeatable: true,\n",
+                    "",
+                    indent = indent + 4
+                ));
+            }
+
+            buffer.push_str(&format!("{:indent$}}},\n", "", indent = indent + 2));
         }
 
-        if names.len() > 1 {
-            buffer.push_str(&format!("{:indent$}name: [", "", indent = indent + 4));
-
-            buffer.push_str(
-                &names
-                    .iter()
-                    .map(|name| format!("\"{}\"", name))
-                    .collect::<Vec<_>>()
-                    .join(", "),
-            );
-
-            buffer.push_str("],\n");
-        } else {
-            buffer.push_str(&format!(
-                "{:indent$}name: \"{}\",\n",
-                "",
-                names[0],
-                indent = indent + 4
-            ));
-        }
-
-        if let Some(data) = option.get_help() {
-            buffer.push_str(&format!(
-                "{:indent$}description: \"{}\",\n",
-                "",
-                escape_string(data),
-                indent = indent + 4
-            ));
-        }
-
-        buffer.push_str(&format!("{:indent$}args: ", "", indent = indent + 4));
-
-        buffer.push_str(&gen_args(option, indent + 4));
-
-        buffer.push_str(&format!("{:indent$}}},\n", "", indent = indent + 2));
+        buffer.push_str(&format!("{:indent$}],\n", "", indent = indent));
     }
 
-    for flag in generator::utils::flags(app) {
-        buffer.push_str(&format!("{:indent$}{{\n", "", indent = indent + 2));
-
-        let mut flags = vec![];
-
-        if let Some(shorts) = flag.get_short_and_visible_aliases() {
-            flags.extend(shorts.iter().map(|s| format!("-{}", s)));
-        }
-
-        if let Some(longs) = flag.get_long_and_visible_aliases() {
-            flags.extend(longs.iter().map(|s| format!("--{}", s)));
-        }
-
-        if flags.len() > 1 {
-            buffer.push_str(&format!("{:indent$}name: [", "", indent = indent + 4));
-
-            buffer.push_str(
-                &flags
-                    .iter()
-                    .map(|name| format!("\"{}\"", name))
-                    .collect::<Vec<_>>()
-                    .join(", "),
-            );
-
-            buffer.push_str("],\n");
-        } else {
-            buffer.push_str(&format!(
-                "{:indent$}name: \"{}\",\n",
-                "",
-                flags[0],
-                indent = indent + 4
-            ));
-        }
-
-        if let Some(data) = flag.get_help() {
-            buffer.push_str(&format!(
-                "{:indent$}description: \"{}\",\n",
-                "",
-                escape_string(data).as_str(),
-                indent = indent + 4
-            ));
-        }
-
-        buffer.push_str(&format!("{:indent$}}},\n", "", indent = indent + 2));
-    }
-
-    buffer.push_str(&format!("{:indent$}],\n", "", indent = indent));
-
     buffer
 }
 
 fn gen_args(arg: &Arg, indent: usize) -> String {
-    if !arg.is_set(ArgSettings::TakesValue) {
+    if !arg.is_takes_value_set() {
         return "".to_string();
     }
 
@@ -221,11 +308,11 @@ fn gen_args(arg: &Arg, indent: usize) -> String {
     buffer.push_str(&format!(
         "{{\n{:indent$}  name: \"{}\",\n",
         "",
-        arg.get_name(),
+        arg.get_id(),
         indent = indent
     ));
 
-    if arg.is_set(ArgSettings::MultipleValues) {
+    if arg.is_multiple_values_set() {
         buffer.push_str(&format!(
             "{:indent$}isVariadic: true,\n",
             "",
@@ -233,7 +320,7 @@ fn gen_args(arg: &Arg, indent: usize) -> String {
         ));
     }
 
-    if !arg.is_set(ArgSettings::Required) {
+    if !arg.is_required_set() {
         buffer.push_str(&format!(
             "{:indent$}isOptional: true,\n",
             "",
@@ -249,27 +336,34 @@ fn gen_args(arg: &Arg, indent: usize) -> String {
         ));
 
         for value in data {
-            buffer.push_str(&format!(
-                "{:indent$}{{\n{:indent$}  name: \"{}\",\n",
-                "",
-                "",
-                value.get_name(),
-                indent = indent + 4,
-            ));
-
             if let Some(help) = value.get_help() {
+                buffer.push_str(&format!(
+                    "{:indent$}{{\n{:indent$}  name: \"{}\",\n",
+                    "",
+                    "",
+                    value.get_name(),
+                    indent = indent + 4,
+                ));
+
                 buffer.push_str(&format!(
                     "{:indent$}description: \"{}\",\n",
                     "",
                     escape_string(help),
-                    indent = indent + 4
+                    indent = indent + 6
+                ));
+
+                buffer.push_str(&format!("{:indent$}}},\n", "", indent = indent + 4));
+            } else {
+                buffer.push_str(&format!(
+                    "{:indent$}\"{}\",\n",
+                    "",
+                    value.get_name(),
+                    indent = indent + 4,
                 ));
             }
-
-            buffer.push_str(&format!("{:indent$}}},\n", "", indent = indent + 4));
         }
 
-        buffer.push_str(&format!("{:indent$}]\n", "", indent = indent + 2));
+        buffer.push_str(&format!("{:indent$}],\n", "", indent = indent + 2));
     } else {
         match arg.get_value_hint() {
             ValueHint::AnyPath | ValueHint::FilePath | ValueHint::ExecutablePath => {
@@ -302,3 +396,19 @@ fn gen_args(arg: &Arg, indent: usize) -> String {
 
     buffer
 }
+
+fn arg_conflicts(cmd: &Command, arg: &Arg) -> Vec<String> {
+    let mut res = vec![];
+
+    for conflict in cmd.get_arg_conflicts_with(arg) {
+        if let Some(s) = conflict.get_short() {
+            res.push(format!("-{}", s));
+        }
+
+        if let Some(l) = conflict.get_long() {
+            res.push(format!("--{}", l));
+        }
+    }
+
+    res
+}
